package com.application.GUI.PopUpWindows;

import com.application.DB.Constants;
import javafx.application.Platform;
import javafx.collections.FXCollections;
import javafx.collections.ObservableList;
import javafx.geometry.Pos;
import javafx.scene.*;
import javafx.scene.control.*;
import javafx.scene.layout.*;
import javafx.stage.*;

import java.time.LocalDateTime;
import java.time.format.DateTimeFormatter;
import java.util.Map;

import static com.application.DB.Settings.*;
import static com.application.DB.DB.getCurrentDrying;
import static com.application.DB.Constants.MAX_USER_INPUT_CHARACTERS;
import static com.application.DB.HelpingFunctions.setLoadedData;
import static com.application.GUI.LineChartFunctionality.*;
import static com.application.Main.*;
import static com.application.DB.DB.setInputParameters;


/**
 * This class handles the popup input window
 *
 * @author Eilert Tunheim, Karin Pettersen, Mads Arnesen
 * @version 1.0
 */
public class InputPopup {


    public static void display() {
        Stage window = new Stage();

        window.initModality(Modality.APPLICATION_MODAL);
        window.setTitle("Input Parameters");

        // Top
        Label inputLabel= new Label("Input");
        inputLabel.setId("inputTop");

        // Center - Input fields
        // Tree species
        Label treeSpeciesInputLabel = new Label("Tree Species");
        treeSpeciesInputLabel.setId("inputLabel");
        ObservableList<String> treeSpecies = FXCollections.observableArrayList(
                 "Furu",
                        "Gran"
        );
        ComboBox<String> treeSpeciesList = new ComboBox<>(treeSpecies);
        treeSpeciesList.setPromptText("Select Tree Species");
        treeSpeciesList.setId("inputDropDownBox");
        treeSpeciesList.setEditable(true);

        // Dimensions
        Label dimensionsInputLabel = new Label("Dimensions");
        dimensionsInputLabel.setId("inputLabel");
        ObservableList<String> dimensions = FXCollections.observableArrayList(
         "47x150",
                "47x175",
                "47x200",
                "47x225",
                "47x250",
                "50x150",
                "50x175",
                "50x200",
                "50x225",
                "50x250"
                );
        ComboBox<String> dimensionsList = new ComboBox<>(dimensions);
        dimensionsList.setPromptText("Select Dimensions");
        dimensionsList.setId("inputDropDownBox");
        dimensionsList.setEditable(true);



        // Sawset
        Label sawsetInputLabel = new Label("Sawset");
        sawsetInputLabel.setId("inputLabel");
        ObservableList<String> sawset = FXCollections.observableArrayList(
                     "1ex",
                            "2ex",
                            "3ex",
                            "4ex"
        );
        ComboBox<String> sawsetList = new ComboBox<>(sawset);
        sawsetList.setPromptText("Select Sawset");
        sawsetList.setId("inputDropDownBox");
        sawsetList.setEditable(true);

        // Moisture
        Label moistureGoalInputLabel = new Label("Moisture Goal");
        moistureGoalInputLabel.setId("inputLabel");
        ObservableList<String> moistureGoal = FXCollections.observableArrayList(
                 "10%",
                        "12%",
                        "14%",
                        "16%",
                        "18%",
                        "20%"
        );
        ComboBox<String> moistureList = new ComboBox<>(moistureGoal);
        moistureList.setPromptText("Select Moisture Goal");
        moistureList.setId("inputDropDownBox");
        moistureList.setEditable(true);


        // Bottom - start button
        Button startButton = new Button("Start");
        startButton.setId("inputButtonStart");

        startButton.setOnAction(e -> {
            // Sets the start time
            Constants.CURRENT_DATE = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(LocalDateTime.now());
            Constants.START_TIME = Constants.CURRENT_DATE;

            // Retrieves the user inputs
            Constants.TREE_SPECIES = treeSpeciesList.getValue();
            Constants.DIMENSIONS = dimensionsList.getValue();
            Constants.SAWSET = sawsetList.getValue();
            if (moistureList.getValue() != null && moistureList.getValue().contains("%")) {
                Constants.MOISTURE_GOAL = moistureList.getValue().replace("%", "");
            } else {
                Constants.MOISTURE_GOAL = moistureList.getValue();
            }

            boolean err = false;

            // If the input is null, sets the value to be empty
            if (treeSpeciesList.getValue() == null) {
                Constants.TREE_SPECIES = "";
            } else if (treeSpeciesList.getValue().length() > MAX_USER_INPUT_CHARACTERS) {
                NotificationPopUp.displayNotificationWindow("A maximum of "+MAX_USER_INPUT_CHARACTERS+" characters is allowed!");
                treeSpeciesList.setValue("");
                err = true;
            }

            if (dimensionsList.getValue() == null) {
                Constants.DIMENSIONS = "";
            }  else if (dimensionsList.getValue().length() > MAX_USER_INPUT_CHARACTERS) {
                NotificationPopUp.displayNotificationWindow("A maximum of "+MAX_USER_INPUT_CHARACTERS+" characters is allowed!");
                dimensionsList.setValue("");
                err = true;
            }

            if (sawsetList.getValue() == null) {
                Constants.SAWSET = "";
            } else if (sawsetList.getValue().length() > MAX_USER_INPUT_CHARACTERS) {
                NotificationPopUp.displayNotificationWindow("A maximum of "+MAX_USER_INPUT_CHARACTERS+" characters is allowed!");
                sawsetList.setValue("");
                err = true;
            }

            if (moistureList.getValue() == null) {
                Constants.MOISTURE_GOAL = "";
            } else if (moistureList.getValue().length() > MAX_USER_INPUT_CHARACTERS) {
                NotificationPopUp.displayNotificationWindow("A maximum of "+MAX_USER_INPUT_CHARACTERS+" characters is allowed!");
                moistureList.setValue("");
                err = true;
            }




            if (!err) {


                setTreeSpeciesText(Constants.TREE_SPECIES);
                setDimensionsText(Constants.DIMENSIONS);
                setSawsetText(Constants.SAWSET);
                setMoistureGoalText(Constants.MOISTURE_GOAL);

                window.close();




            // Fungerende ny thread!!@@@@@
            // Gather data
            try {
                Thread thread = new Thread(() -> {

                    try {
                        setLoadedData(false);
                        // Henter her data fra databasen
                        Map<Integer, Map<String, Number>> data = setInputParameters();
                        Platform.runLater(() -> {
                            try {
                                loadSingleSeries(data);
                                setLoadedData(true);
                            } catch (Exception ex) {
                                ex.printStackTrace();
                            }
                        });
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }
                );
                thread.setDaemon(true);
                thread.interrupt();
                thread.join();
                //Platform.exit();
                thread.start();

            } catch (Exception ex) {
                ex.printStackTrace();
            }

            // Retrieve data for current drying period
            try {
                Thread liveDataThread = new Thread(() -> {
                    try {
                        while (!Constants.IS_FINISHED) {

                            Constants.STOP_TIME = DateTimeFormatter.ofPattern("yyyy-MM-dd HH:mm:ss").format(LocalDateTime.now());
                            Map<String, Number> data = getCurrentDrying();
                            Platform.runLater(() -> {
                                try {
                                    loadLiveData(data);
                                } catch (Exception ex) {
                                    ex.printStackTrace();
                                }
                            });
                            Thread.sleep(1000L * NUMBER_OF_SECONDS_LIVE_DATA);
                        }
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }
                );
                liveDataThread.setDaemon(true);
                liveDataThread.interrupt();
                liveDataThread.join();
                //Platform.exit();
                liveDataThread.start();

            } catch (Exception ex) {
                ex.printStackTrace();
            }

        }
                }

        );

/*
                    // Fungerende ny thread!!@@@@@
                    try{
                        Thread thread = new Thread(new Runnable() {

                            @Override
                            public void run() {
                                try {
                                    // Henter her data fra databasen
                                    loadSingleSeries(setInputParameters());
                                    //loadSingleSeries();
                                    //loadMultipleSeries();
                                } catch (Exception ex) {
                                    ex.printStackTrace();
                                }
                            }
                        });
                        thread.start();
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                }

                );
        */





/*
            class KwhThread implements Runnable  {

                Map<Integer, Map<String, Number>> dataVariable;
                private volatile boolean finished;

                @Override
                public void run() {
                    try{
                        dataVariable = setInputParameters();
                    } catch (Exception ex) {
                        ex.printStackTrace();
                    }
                    finished = true;

                    synchronized (this){
                        this.notify();
                    }
                }

                public Map<Integer, Map<String, Number>> getDataVariable() throws InterruptedException {

                    synchronized (this){
                        if(!finished)
                        this.wait();
                    }
                    return dataVariable;
                }
            }

            KwhThread kwhThread = new KwhThread();

            Thread thread = new Thread(kwhThread);
            thread.setName("GetKwhThread");
            thread.start();

            try {
                loadSingleSeries(kwhThread.getDataVariable());
            } catch (Exception ex) {
                ex.printStackTrace();
            }


 */

/*


 */


        VBox layout = new VBox(10);
        layout.getChildren().addAll(inputLabel, treeSpeciesInputLabel, treeSpeciesList, dimensionsInputLabel, dimensionsList,
                sawsetInputLabel, sawsetList, moistureGoalInputLabel, moistureList, startButton);
        layout.setAlignment(Pos.CENTER);

        Scene scene = new Scene(layout, 600, 500);
        scene.getStylesheets().add(InputPopup.class.getResource("/com.application/CSS/styleSheet.css").toExternalForm());
        window.setScene(scene);
        window.showAndWait();
    }
}
